# Probabilità

## Concetto di probabilità

Consideriamo un insieme S di eventi detto "spazio campionario" e due eventi
casuali A e B. Essi sono soggetti agli assiomi di Kolmogorov:

$\forall A \subseteq S,  0 \leqslant P(A) \leqslant 1$  
$P(S) = 1$  
$A \cap B = \varnothing \Longrightarrow P(A \cup B) = P(A) + P(B)$

Si definisce probabilità condizionata di A dato B:

$$
  P(A|B) = \frac{P(A \cap B)}{P(B)}
$$

Se due eventi sono indipendenti, allora vale che:

$$
  P(A \cap B) = P(A) \cdot P(B) \Longrightarrow
  P(A|B) = \frac{P(A) \cdot P(B)}{P(B)} = P(A)
$$

Esistono diversi approcci per definire la probabilità:

- approccio frequentista:

  $$
    P(A) = \lim_{n \rightarrow +\infty} \frac{\#A}{n}
  $$

  ma c'è il problema che è impossibile fare un numero infinito di tentativi.
- approccio bayesiano:  
  utilizzare il teorema di Bayes, per esempio:

  $$
    P(\text{teoria|dati}) \propto P(\text{dati|teoria}) \cdot P(\text{teoria})
  $$
  si tratta quindi di un approccio "soggettivo", almeno per l'ultimo termine
  dell'equazione qua sopra, perché la probabilità della validità della teoria
  può essere data sulla base di ragionamenti e osservazioni riguardanti il
  fenomeno di cui si sta parlando.

\begin{tikzpicture}
  \draw [thick, pink] (0,0) -- (1,0);
  \draw [red] (1.5,0) circle [radius=0.1];
  \draw [thick, pink] (2,0) -- (3,0);
  \draw [red] (3.5,0) circle [radius=0.1];
  \draw [thick, pink] (4,0) -- (5,0);
  \draw [red] (5.5,0) circle [radius=0.1];
  \draw [thick, pink] (6,0) -- (7,0);
  \draw [red] (7.5,0) circle [radius=0.1];
  \draw [thick, pink] (8,0) -- (9,0);
  \draw [red] (9.5,0) circle [radius=0.1];
  \draw [thick, pink] (10,0) -- (11,0);
  \draw [red] (11.5,0) circle [radius=0.1];
  \draw [thick, pink] (12,0) -- (13,0);
  \draw [red] (13.5,0) circle [radius=0.1];
  \draw [thick, pink] (14,0) -- (15,0);
\end{tikzpicture}

### Esempio di probabilità condizionata

Si osservano dei decadimenti e se ne misurano indipendentemente $N_a$ e $N_b$;
in comune ne sono stati misurati $N_{ab}$. Possiamo stimare il numero totale
di eventi $N$ e l'efficienza totale $\epsilon$.

$$
  P(a) = \frac{N_a}{N} \hspace{100pt} P(b) = \frac{N_b}{N}
$$
$$
  P(ab) = \frac{N_{ab}}{N} = P(a) \cdot P(b) = \frac{N_a \cdot N_b}{N^2}
  \Longrightarrow N = \frac{N_a \cdot N_b}{N_{ab}}
$$
$$
  \epsilon = P(a \cup b) = P(a) + P(b) -P(a \cap b) =
  \frac{N_a + N_b - N_{ab}}{N}
$$

\begin{tikzpicture}
  \draw [thick, pink] (0,0) -- (1,0);
  \draw [red] (1.5,0) circle [radius=0.1];
  \draw [thick, pink] (2,0) -- (3,0);
  \draw [red] (3.5,0) circle [radius=0.1];
  \draw [thick, pink] (4,0) -- (5,0);
  \draw [red] (5.5,0) circle [radius=0.1];
  \draw [thick, pink] (6,0) -- (7,0);
  \draw [red] (7.5,0) circle [radius=0.1];
  \draw [thick, pink] (8,0) -- (9,0);
  \draw [red] (9.5,0) circle [radius=0.1];
  \draw [thick, pink] (10,0) -- (11,0);
  \draw [red] (11.5,0) circle [radius=0.1];
  \draw [thick, pink] (12,0) -- (13,0);
  \draw [red] (13.5,0) circle [radius=0.1];
  \draw [thick, pink] (14,0) -- (15,0);
\end{tikzpicture}

## Teorema di Bayes

$$
  P(A|B) = \frac{P(A \cap B)}{P(B)}
  \hspace{100pt}
  P(B|A) = \frac{P(B \cap A)}{P(A)}
$$
$$
  P(A|B) = \frac{P(B|A) \cdot P(A)}{P(B)}
$$

Dal teorema di Bayes è possibile dedurre il teorema della probabilità totale:
consideriamo un insieme $S$ diviso in sottoinsiemi disgiunti $A_i$ la cui
unione dà l'insieme di partenza:

$$
  \cup_i A_i = S
$$

e consideriamo un insieme $B$ anch'esso interno ad $S$:

$$
  B = B \cap S = B \cap (\cup_i A_i) = \cup_i (B \cap A_i)
$$
$$
  \Longrightarrow P(B) = \sum_i P(B \cap A_i)
$$
$$
  \Longrightarrow P(B) = \sum_i P(B|A_i)P(A_i)
$$

da cui, per uno specifico insieme $A_j$, attraverso il teorema di Bayes:

$$
  P(A_j|B) = \frac{P(B|A_j) \cdot P(A_j)}{\sum_i P(B|A_i)P(A_i)}
$$

\begin{tikzpicture}
  \draw [thick, pink] (0,0) -- (1,0);
  \draw [red] (1.5,0) circle [radius=0.1];
  \draw [thick, pink] (2,0) -- (3,0);
  \draw [red] (3.5,0) circle [radius=0.1];
  \draw [thick, pink] (4,0) -- (5,0);
  \draw [red] (5.5,0) circle [radius=0.1];
  \draw [thick, pink] (6,0) -- (7,0);
  \draw [red] (7.5,0) circle [radius=0.1];
  \draw [thick, pink] (8,0) -- (9,0);
  \draw [red] (9.5,0) circle [radius=0.1];
  \draw [thick, pink] (10,0) -- (11,0);
  \draw [red] (11.5,0) circle [radius=0.1];
  \draw [thick, pink] (12,0) -- (13,0);
  \draw [red] (13.5,0) circle [radius=0.1];
  \draw [thick, pink] (14,0) -- (15,0);
\end{tikzpicture}

### Esempio di probabilità totale: segnale e fondo

Un rilevatore misura segnale e fondo con relative efficienze $P(R|S)$ e
$P(R|F)$. Se sono note a priori la probabilità di segnale e di fondo
$P(S)$ e $P(F)$, allora si può risalire alla probabilità, data una
misurazione, di aver misurato il segnale:

$$
  P(S|R) = \frac{P(R|S) \cdot P(S)}{P(R|S) \cdot P(S) + P(R|F) \cdot P(F)}
$$

\begin{tikzpicture}
  \draw [thick, pink] (0,0) -- (1,0);
  \draw [red] (1.5,0) circle [radius=0.1];
  \draw [thick, pink] (2,0) -- (3,0);
  \draw [red] (3.5,0) circle [radius=0.1];
  \draw [thick, pink] (4,0) -- (5,0);
  \draw [red] (5.5,0) circle [radius=0.1];
  \draw [thick, pink] (6,0) -- (7,0);
  \draw [red] (7.5,0) circle [radius=0.1];
  \draw [thick, pink] (8,0) -- (9,0);
  \draw [red] (9.5,0) circle [radius=0.1];
  \draw [thick, pink] (10,0) -- (11,0);
  \draw [red] (11.5,0) circle [radius=0.1];
  \draw [thick, pink] (12,0) -- (13,0);
  \draw [red] (13.5,0) circle [radius=0.1];
  \draw [thick, pink] (14,0) -- (15,0);
\end{tikzpicture}

### Esempio di probabilità totale: AIDS 

$$
  P(\text{AIDS}) = 0.001 \hspace{50pt} P(\text{no AIDS}) = 0.999
$$
$$
  P(\text{+|AIDS}) = 0.98 \hspace{50pt} P(\text{-|AIDS}) = 0.02
$$
$$
  P(\text{+|no AIDS}) = 0.03 \hspace{50pt} P(\text{-|no AIDS}) = 0.97
$$

Quindi se il test è positivo, la probabilità di avere davvero preso l'AIDS è:

$$
  P(\text{AIDS|+}) = \frac{P(\text{+|AIDS}) \cdot P(\text{AIDS})}
  {P(\text{+|AIDS}) \cdot P(\text{AIDS}) + P(\text{+|no AIDS}) \cdot
  P(\text{no AIDS})} = 0.032
$$

\begin{tikzpicture}
  \draw [thick, pink] (0,0) -- (1,0);
  \draw [red] (1.5,0) circle [radius=0.1];
  \draw [thick, pink] (2,0) -- (3,0);
  \draw [red] (3.5,0) circle [radius=0.1];
  \draw [thick, pink] (4,0) -- (5,0);
  \draw [red] (5.5,0) circle [radius=0.1];
  \draw [thick, pink] (6,0) -- (7,0);
  \draw [red] (7.5,0) circle [radius=0.1];
  \draw [thick, pink] (8,0) -- (9,0);
  \draw [red] (9.5,0) circle [radius=0.1];
  \draw [thick, pink] (10,0) -- (11,0);
  \draw [red] (11.5,0) circle [radius=0.1];
  \draw [thick, pink] (12,0) -- (13,0);
  \draw [red] (13.5,0) circle [radius=0.1];
  \draw [thick, pink] (14,0) -- (15,0);
\end{tikzpicture}

### Approccio bayesiano con una teoria

Cominciamo com un esempio: con un esperimento è stata misurata la massa di
un elettrone e sono stati trovati i valori {$m_i$}, di cui il valore medio
è $m_e = \SI{520 \pm 10}{KeV}$. Si assume quindi che il valore vero sia
compreso tra \SI{510}{KeV} e \SI{530}{KeV} con una probabilità del 68% data
dal confidence level. Per cui, la probabilità che la massa vera sia proprio
\SI{520}{KeV} è data da:

$$
  P(m_e|{m_i}) = \frac{}{}
$$

\textcolor{red}{**rivedere negli appunti questa formula**}

Quando si fa una misurazione, si misurano $N$ valori e si calcolano il valore
medio $\bar{x}$ e la deviazione standard $\sigma$ e si dice che il risultato è:

$$
  \mu = \bar{x} \pm \frac{\sigma}{\sqrt{N}}
$$

e di solito lo si interpreta dicendo che:

$$
  P \left( \bar{x} - \frac{\sigma}{\sqrt{N}} \leqslant \mu \leqslant
  \bar{x} + \frac{\sigma}{\sqrt{N}} \right) = 68\%
$$

ma in realtà quello che sappiamo è solo che:

$$
  P \left( \mu - \frac{\sigma_{true}}{\sqrt{N}} \leqslant \bar{x} \leqslant
  \mu + \frac{\sigma_{true}}{\sqrt{N}} \right) = 68\%
$$

**\textcolor{red}{WHAT?!?}**
